## This R script demonstrates how to calculate and plot an exposure-response curve based on annoyance responses
## on a 05-point ICBEN scale for a simulated cutoff point of 72.73% (instead of a cutoff point at 60%).
## For comparison, the curve for the 72.73% cutoff on the 11-point scale is also calculated and plotted.
## A working data source for this script is provided in the file 'Sample_Data.csv'.
## The script does not need additional packages to run. It was tested with R Version 3.5.1
##
## Note: Annoyance score on the 5-point scale in the file 'Sample_Data.csv' has values 0,1,2,3,4
## (corresponding to the verbal scale point labels: 0="not at all"; 1="slightly"; 2="moderately", 3="very", 4="extremely")
## Annoyance score on the 11-point scale in the file 'Sample_Data.csv' has values 0,1,2,3,4,5,6,7,8,9,10


## Clear memory, remove plot windows (if any), set options
remove(list=ls())
dev.off(2); dev.off(3); dev.off(4); dev.off(5); dev.off(6); dev.off(7); dev.off(8); dev.off(9); dev.off(10)
windows.options(width=10, height=8)


### General part
### ############

## Read-in original data from file (adjust the full path if necessary) 
DataOrig <- read.table("C:/Sample_Data.csv", sep = ";", header = TRUE)

## Adjust names of variables and check structure of the data
# AN05: Score on the 5-point scale from 0...4
# AN11: Score on the 11-point scale from 0...11
names(DataOrig)[3:4] <- c("AN05", "AN11") #AN05=Annoyance score on the 5-pt scale; AN11=Annoyance score on the 11-pt scale
str(DataOrig)
nrow(DataOrig) #there are 1000 cases in the original data file

## Determine exposure range in the data file
minLden<-floor(min(DataOrig$Lden)); minLden
maxLden<-ceiling(max(DataOrig$Lden)); maxLden

## Define constants:
z <- qnorm(1-0.05/2, mean=0, sd=1)      #z-value of the standard normal distribution for a 95% confidence interval (will be used multiple times later)
Fabove=0.36                             #Proability of a person with value 3 (i.e. 'very') in the variable AN05, to be HA, for a cutoff of 72.73% (expectation-free value)
                                        #This value might by adapted if empirical results suggest it to be lower or higher than 0.36
nvery <- sum(DataOrig$AN05==3); nvery   #Number of 'very' annoyed (numerical value 3) persons in the original dataset
# Run some checks
Fabove * nvery                          #about 83 respondents (out of 1000) are both 'very' annoyed and considered HA=1
(1-Fabove) * nvery                      #consequently, about 148 persons (out of 1000) are 'very' annoyed, but considered HA=0

### Random sampling part
### ######################

## Step 1: create list (called 'HA') of N identical vectors and assign the value 1 (TRUE) to all responses where AN05=4
N = 500
HA <- rep(list(HA = as.numeric(DataOrig$AN05 == 4)), N) #declare the list object

## Step 2: Randomly assign HA=0 or HA=1, according to the values assumed for Fabove
set.seed(123)                           #This statement makes multiple runs of this program comparable (this must be run every time before a random process is 
                                        #evoked to get the same analysis results every time)
for (i in 1:N) {
  HA[[i]][DataOrig$AN05==3] <- sample(0:1, size=nvery, replace=TRUE, prob=c(1-Fabove, Fabove)) #randomly sample HA=0 or HA=1, according to Fabove
}

summary(sapply(HA, FUN = function(x) sum(x[DataOrig$AN05 == 3]))) #distribution statistis
# We now have N vectors contained in the list 'HA' that reflect the probability of HA for a cutoff of 72.73%
# We can now start with the statistical modelling


### Modelling part
### ##############

## Step 3: Create list 'ModResults' that will contain the estimates and std. errors of N glm-fits, based on
##         the N variations in the list 'HA'
ModResults <- vector(mode = "list", length = N)       #Declare list object
for (i in 1:N) {
    fit <- glm(HA[[i]] ~ 1 + DataOrig$Lden, family = binomial(link = "logit"))
    ModResults[[i]] <- c(coef(fit), VCov = vcov(fit)) #Save coeffients and VC-matrix of each of i...N fits
}

# Run some checks
head(ModResults) #shows the first 6 entries in the list

## Step 4: From the list 'ModResults' determine the average model (for HA=1 on the 5-point scale for a cutoff of 72.73%)
AverageModel <- apply(simplify2array(ModResults), MARGIN = 1, FUN = mean) #for each column in ModResults, calculate the mean
AverageModel
# What do the 6 figures stand for?
# "(Intercept)"=coefficient of intercept, "DataOrig$Lden"=coefficient of Lden, "VCov1"=variance of intercept, "VCov2"="VCov3"=covariance of intercept and Lden, "VCov4"=variance of Lden
# With these parameters, we could now already plot the logistic curve
# However, we also have to calculate the correct confidence intervals, which we do in the next step

## Step 5: Calculate confidence intervals that account for within AND beetween variance

# Calculate within (each model) and between (all models) variance
var.within <- matrix(apply(simplify2array(ModResults)[3:6,], MARGIN = 1, FUN = mean), nrow = 2, byrow = TRUE)
var.within
var.between <- cov(cbind(simplify2array(ModResults)[1,], simplify2array(ModResults)[2,]))
var.between

# Calculate total variances taking the variability
# within each of N models and across N models into account
# (For the ease of use, we use asymptotic degrees of freedom)

# total variance of intercept
var.tot.intercept <- var.within[1, 1] + (1 + 1 / N) * var.between[1, 1]
var.tot.intercept

# total variance of Lden
var.tot.Lden <- var.within[2, 2] + (1 + 1 / N) * var.between[2, 2]
var.tot.Lden

# total covariance
cov.tot <- var.within[1, 2] + (1 + 1 / N) * var.between[1, 2]
cov.tot

## Step 6: Prepare plot
## Values on the x-axis for the plot. Here, we use a range from minLden (30 dB) to maxLden (80 dB)
xvalues <- seq(min(minLden), max(maxLden), by=1.0); xvalues

## Calculate standard erros (SEs) for each value of x. SEs are needed to calculate the pertinent confidence intervals
SEs <- sqrt(var.tot.intercept + xvalues^2 * var.tot.Lden + 2 * xvalues * cov.tot); SEs

## Create vectors for the curves: transform from logit scale to probability scale and raise figure to reflect percents [%]
y0 <- matrix(cbind(1, xvalues), ncol = 2) %*% (AverageModel[1:2])   #curve on the logit scale
Curve_ModSim <- plogis(y0) *100                                     #curve on the probability scale
lowerCI_ModSim <- plogis(y0 - z * SEs) *100                         #lower bound of confidence interval on the probability scale
upperCI_ModSim <- plogis(y0 + z * SEs) *100                         #upper bound of confidence interval on the probability scale

# Check
cbind(lowerCI_ModSim, Curve_ModSim, upperCI_ModSim)


### Plotting part
### #############

## For comparing the 11-point curve (the 'reference') with the 5-point curve with the simulated cutoff
## we need to estimate the model for the 11-point scale first:
# Set HA=1 for those that score higher or equal 8 on the 11-point scale in the original dataset and run the model
DataOrig$HA11 <- ifelse(DataOrig$AN11>=8, 1, 0)     #corresponds to a cutoff point at 72.73%
ModRef <- glm("HA11 ~ Lden", data=DataOrig, family="binomial"(link="logit"))
# Get the values for the curve and the confidence intervals using the 'predict' function
PredictionModRef <-  predict(ModRef,
                             newdata = data.frame(HA11=0, Lden=xvalues),
                             type="response",        #Predictions given on the probability scale and not on the logit scale!!
                             se.fit = TRUE)          #Also estimate standard errors
# Create vectors for the curve for the lower and upper confidence interval bounds
Curve_ModRef <- PredictionModRef$fit*100
lowerCI_ModRef <- (PredictionModRef$fit - z * PredictionModRef$se.fit)*100
upperCI_ModRef <- (PredictionModRef$fit + z * PredictionModRef$se.fit)*100

## Step 7: create plot
## Define the options for the plot
dev.new()
par(oma=c(1.2,1,1,1), mar=c(5,6,4,1), mfrow=c(1,1))
linecol1=rgb(52,149,235,alpha=255, maxColorValue=255) #Color for Curve_ModRef 
cicol1=rgb(52,149,235,alpha=50, maxColorValue=255)    #Color for CI_ModRef 
linecol2=rgb(164,27,50,alpha=255, maxColorValue=255)  #Color for Curve_ModSim
cicol2=rgb(164,27,50,alpha=50, maxColorValue=255)     #Color for CI_ModSim

## Create plot
plot(main = "Comparison of curves", 
          xvalues,
          Curve_ModRef,
          type="n",
          ylim=(c(0,60)),
          lwd=3, lty=1,
          xlab="Lden [dB]", ylab="%HA",
          cex.main=1.9, cex.axis = 1.9, cex.lab = 1.9, font.lab=2, font=2, mgp = c(3.8,  1.4,  0))


## Draw confidence bands, gridlines, exposure-response curves, and legend
polygon(c(rev(xvalues), xvalues), c(rev(upperCI_ModRef), lowerCI_ModRef), col=cicol1, border=NA)
polygon(c(rev(xvalues), xvalues), c(rev(upperCI_ModSim), lowerCI_ModSim), col=cicol2, border=NA)
grid (NULL,NULL, lty = 1, col = "darkgray")
lines(xvalues, Curve_ModRef, lwd=3, lty=1, col=linecol1)
lines(xvalues, Curve_ModSim, lwd=3, lty=1, col=linecol2)
legend(x=30, y=60, legend=c("11-pt scale w. cutoff point 72.73% (reference)",                             
                            paste0("05-pt scale w. simulated cutoff point 72.73% for Fabove = ", Fabove)),
                            col=c(linecol1, linecol2), lty=1, lwd=4, cex=1.2, bty="o", bg="cornsilk")


### END



